/*====================================================================
* Project:
* Developed using:
* Function: getting started sample with ASC0, interrupt and timer
*
* Copyright HighTec EDV-Systeme GmbH 1982-2007
*====================================================================*/

#include <stdio.h>
#include <machine/wdtcon.h>
#include <machine/intrinsics.h>

#include <tc1796b/scu.h>
#include <tc1796b/port1-struct.h>

#include "rs232.h"
#include "timer.h"

/* TriBoard TC1796.100 : P1.15 --> LED D401 */
#define TOGGLE_LED		15

/* crystal on V100 is 16MHz */
/*#define CRYSTAL		16000000U*/
/* crystal on V101 is 20MHz */
#define CRYSTAL		20000000U


/************ MAIN Procedure *****************************************************************************************/

int main(void)
{
	unsigned int	K_Div, N_Div, P_Div;
	unsigned int	Hour, Min, Sec, OldSec;
	unsigned int	frequency;
	char			msg[50];
	PORT1_t			*port = (PORT1_t *)(P1_BASE);
	PLL_CLC_t_nonv	pllclc;

	/* set speed to 128 MHz with 16Mhz Crystal or 150 MHz with 20MHz Crystal */
#if (CRYSTAL == 16000000U)
	pllclc.reg = 0;
	pllclc.bits.NDIV = 40-1;
	pllclc.bits.KDIV =  4-1;
	pllclc.bits.PDIV =  1-1;
	pllclc.bits.VCOSEL =  2;
#else
	pllclc.reg = 0;
	pllclc.bits.NDIV = 30-1;
	pllclc.bits.KDIV =  4-1;
	pllclc.bits.PDIV =  1-1;
	pllclc.bits.VCOSEL =  2;
#endif
	unlock_wdtcon();
	PLL_CLC = pllclc;
	lock_wdtcon();

	/* get frequency */
	pllclc = PLL_CLC;

	N_Div = pllclc.bits.NDIV + 1;
	K_Div = pllclc.bits.KDIV + 1;
	P_Div = pllclc.bits.PDIV + 1;

	if (pllclc.bits.BYPPIN)
		frequency = CRYSTAL;
	else frequency = CRYSTAL * N_Div / (K_Div * P_Div);

	/* initialize ASC0 dependant from frequency */
	if (!pllclc.bits.SYSFS)  init_rs232(frequency/2);
	else init_rs232(frequency);

	/* enable interrupts */
	_enable();

	/* send strings via ASC0 */
	message("\r\n\r\n\r\n");
	message("Hello World! (HIGHTEC version using GNU tools)\r\n\r\n");
	message("I'am the TriBoard with TC1796\r\n");
	message("developed at\r\n");
	message("INFINEON Technologies AG in Munich\r\n");
	message("Department AI MC AE\r\n\r\n");
	message("Thomasiusplatz\r\n");
	message("D-81541 Munich\r\n");
	message("Tel.:+49-89-234-0\r\n");
	message("Fax.:+49-89-234-81785\r\n\r\n");
	message("If you have questions to this board or to TriCore CPU,\r\n");
	message("see the manuals on the TriBoard CD.\r\n");
	message("This is the getting started demo.\r\n");
	message("Have fun working with me!\r\n\r\n");
	sprintf(msg, "The CPU   running at %d Hz\r\n", frequency);
	message(msg);

	if (!pllclc.bits.SYSFS) frequency /= 2;
	sprintf(msg, "The Board running at %d Hz\r\n\r\n", frequency);
	message(msg);

	Sec = 0;
	Min = 0;
	Hour = 0;
	OldSec = 59;

	/* initialize PORT1.15 */
	port->IOCR12.bits.PC3 = 0x8;	/* OUT_PPGPIO */;
	/* initialize Port0.7 */

	/* initialize timer dependant from frequency */
	init_timer(frequency);

	while (1)
	{
		/* calculate the actual time */
		Sec = Ticks/1000;
		Sec = Sec%60;
		if (Sec == OldSec) continue;
		OldSec = Sec;
		Min = Ticks/60000;
		Min = Min%60;
		Hour = Ticks/3600000;
		Hour = Hour%24;
		/* send the new time via ASC0 */
		sprintf(msg, "\rrunning since: %02dh:%02dmin:%02dsec", Hour, Min, Sec);
		message(msg);
		/* toggle the LED */
		port->OMR.reg = ((1 << TOGGLE_LED)<<16) | (1 << TOGGLE_LED);
	}

	return 0;
}
